#!/usr/bin/perl -w

# Copyright 2013, 2014, 2017, 2020 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


# Print a tikz picture showing how the UArD tree area copies out to 3 parts
# U,A,D, with dashed parallelograms in between.

use 5.010;
use strict;
use List::Util 'max','min';
use Math::PlanePath::PythagoreanTree;

use FindBin;
use lib $FindBin::Bin;
use MyPrintwrap ':all';
use MyTriples 'U_2x2','A_2x2','D_2x2';


# Region printed is to $depth copied out 3 times.
#
my $depth = 1;


printwrap("  % generated by tools/$FindBin::Script\n");

my $path = Math::PlanePath::PythagoreanTree->new(coordinates => 'PQ',
                                                 tree_type => 'UArD',
                                                 digit_order => 'LtoH');
my @vecs;
foreach my $n (($path->tree_depth_to_n($depth+1)
                .. $path->tree_depth_to_n_end($depth+1)),
               reverse($path->tree_depth_to_n($depth)
                       .. $path->tree_depth_to_n_end($depth))) {
  my ($x,$y) = $path->n_to_xy($n);
  push @vecs, Math::Matrix->new([$x],[$y]);
}

foreach my $m (U_2x2(), A_2x2(), D_2x2()) {
  printwrap_indent("  ");
  printwrap("\\draw\n");
  printwrap_indent("    ");
  foreach my $i (0 .. $#vecs) {
    my $v = $vecs[$i];
    my $join = ($i == 0 ? '' : "--");
    my $str = mvec_to_str($m*$v);
    printwrap("$join($str)");
  }
  printwrap("--cycle;\n");
}

foreach my $n ($path->tree_depth_to_n_end($depth),
               $path->tree_depth_to_n_end($depth+1)) {
  my ($x,$y) = $path->n_to_xy($n);
  my $v = Math::Matrix->new([$x],[$y]);
  my $uv = U_2x2() * $v;
  my $av = A_2x2() * $v;
  print "  \\filldraw[dashed,ultra thin] (",
    mvec_to_str($uv),") circle -- (",mvec_to_str($av),") circle;\n";
}

foreach my $n ($path->tree_depth_to_n($depth),
               $path->tree_depth_to_n($depth+1)) {
  my ($x,$y) = $path->n_to_xy($n);
  my $v = Math::Matrix->new([$x],[$y]);
  my $dv = D_2x2() * $v;
  my $av = A_2x2() * $v;
  print "  \\filldraw[dashed,ultra thin] (",
    mvec_to_str($dv),") circle -- (",mvec_to_str($av),") circle;\n";
}

print "  % end generated\n";
exit 0;
